import os
import logging
import sys
import argparse
from pathlib import Path
from mlfix import fix


def configure_logging():
    level = os.environ.get("LOG_LEVEL", "error").upper()
    logging.basicConfig(level=logging.getLevelName(level))


def _main_parser() -> argparse.ArgumentParser:
    """
    main application parser
    """
    p = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )

    p.add_argument("path_to_store", help="Local path to the artifact store to fix")

    p.add_argument(
        "--mlruns-name",
        type=str,
        default="mlruns",
        help="Name of the mlruns folder (default: %(default)s)",
    )

    return p


def cmd_fix(ns: argparse.Namespace):
    path = Path(ns.path_to_store).resolve().absolute()
    mlruns_name = ns.mlruns_name
    print(
        f"fixing artifact store in path: '{path}', former mlruns name: '{mlruns_name}'..."
    )
    fix.fix_meta(path, mlruns_name)
    print("done!")


def run():
    configure_logging()
    if sys.version_info < (3, 6, 0):
        sys.stderr.write("You need python 3.6 or later to run this script\n")
        sys.exit(1)

    parser = _main_parser()
    namespace = parser.parse_args()
    if namespace.path_to_store is not None:
        cmd_fix(namespace)
    else:
        parser.print_help()



from pathlib import Path
import yaml
from typing import Dict, Any
from mlfix.path_utils import (
    find_file,
    last_index_where,
    uri_to_path,
)

FILENAMES = ["meta.yaml"]
KEYS_TO_FIX = ["artifact_location", "artifact_uri"]


def _fix_path_in_uri(uri: str, mlruns_name: str, new_path_to_store: Path) -> str:
    """
    returns fixed uri
    """
    path = uri_to_path(uri)
    path_parts = list(path.parts)
    im = last_index_where(path_parts, lambda x: x == mlruns_name)
    assert im >= 0
    # create new path
    new_path = new_path_to_store.resolve().absolute().joinpath(*path_parts[im + 1 :])
    # to uri
    return new_path.as_uri()


def fix_meta(path_to_store: Path, mlruns_name: str) -> bool:
    """
    path_to_store - path to artifact store to fix
    mlruns_name - name of the former mlruns folder (the one present in yaml files)
    returns True if any of the files was changed
    """
    assert (
        path_to_store.exists() and path_to_store.is_dir()
    ), f"path_to_store: {path_to_store.resolve()} must be a real folder"
    # find files
    files_to_fix = []
    for fn in FILENAMES:
        files_to_fix.extend(find_file(path_to_store, fn))

    # read each file, replace found key with fixed path and overwrite file
    changed = False
    for fp in files_to_fix:
        with open(fp, "r+") as f:
            contents: Dict[str, Any] = yaml.full_load(f)
            changed = False
            for k in KEYS_TO_FIX:
                if k in contents:
                    contents[k] = _fix_path_in_uri(
                        contents[k], mlruns_name, path_to_store
                    )
                    changed = True
            if changed:
                # save new version of this file
                f.seek(0)
                yaml.dump(contents, f)
    return changed



import os
from pathlib import Path
from urllib.parse import unquote, urlparse
from typing import Any, Callable, List


def find_file(root: Path, name: str) -> List[Path]:
    """Finds files of the specified name in the specified path (recursive)

    Args:
        root (Path): path to search in
        name (str): name to search for (supports globs)

    Returns:
        List[Path]: list of full paths to matching files
    """
    res = []
    for path in root.rglob(name):
        pr = path.resolve()
        if pr.is_file():
            res.append(pr)
    return res


def last_index_where(l: List[Any], where: Callable[[Any], bool]) -> int:
    idx = -1
    for i, v in enumerate(l):
        if where(v):
            idx = i
    return idx


def uri_to_path(uri: str) -> Path:
    p = urlparse(uri)
    final_path = os.path.abspath(os.path.join(p.netloc, unquote(p.path)))
    return Path(final_path)



import pathlib
from setuptools import setup, find_packages
from mlfix import __version__

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

requirements = [
    "pyyaml == 5.4.1",
]

setup(
    name="mlfix",
    version=__version__,
    url="https://github.com/konradmalik/mlfix.git",
    author="Konrad Malik",
    author_email="konrad.malik@gmail.com",
    license="MIT",
    description="mlflow artifact store migration fix tool",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        "console_scripts": ["mlfix = mlfix.__main__:main"],
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
